from setuptools import setup, find_packages

setup(
    name='publickey',
    version='1.0.3',
    packages=find_packages(),
    long_description='Cryptography standards of the CIS countries',
)

setup(
    name='symmetric',
    version='1.0.3',
    packages=find_packages(),
    long_description='Cryptography standards of the CIS countries',
)
